@@ -1,22 +1,20 @@
-# Reader for py4DSTEM v0.7 - v0.8 files
+# Reader for py4DSTEM v0.9 - v0.11 files
 
 import h5py
 import numpy as np
-from os.path import splitext
+from os.path import splitext, exists
 from .read_utils import is_py4DSTEM_file, get_py4DSTEM_topgroups, get_py4DSTEM_version, version_is_geq
-from .read_utils_v0_7 import get_py4DSTEM_dataobject_info
+from .read_utils_v0_9 import get_py4DSTEM_dataobject_info
 from ...datastructure import DataCube, CountedDataCube, DiffractionSlice, RealSlice
 from ...datastructure import PointList, PointListArray
 from ....process.utils import tqdmnd
+from ..metadata import metadata_from_h5
 
-def read_v0_7(fp, **kwargs):
+def read_v0_9(fp, **kwargs):
     """
-    File reader for files written by py4DSTEM v0.7 or v0.8.  Precise behavior is detemined by which
+    File reader for files written by py4DSTEM v0.9-0.11.  Precise behavior is detemined by which
     arguments are passed -- see below.
 
-    ***NOTE: this function has not yet been tested on all legacy py4DSTEM formats. Please report
-    any problems by filing an issue on our github!
-
     Accepts:
         filepath    str or Path     When passed a filepath only, this function checks if the path
                                     points to a valid py4DSTEM file, then prints its contents to screen.
@@ -53,6 +51,7 @@ def read_v0_7(fp, **kwargs):
                                     will return a length two tuple, the first element being a list of 3
                                     DataObject instances and the second a MetaData instance.
     """
+    assert(exists(fp)), "Error: specified filepath does not exist"
     assert(is_py4DSTEM_file(fp)), "Error: {} isn't recognized as a py4DSTEM file.".format(fp)
 
     # For HDF5 files containing multiple valid EMD type 2 files, disambiguate desired data
@@ -72,79 +71,92 @@ def read_v0_7(fp, **kwargs):
             return None,None
 
     version = get_py4DSTEM_version(fp, tg)
-    assert(version == (0,7,0)), "File must be v0.7.0."
+    assert(version_is_geq(version,(0,9,0)) and not version_is_geq(version,(0,12,0))), (
+                                                               "File must be v0.9-0.11")
     _data_id = 'data_id' in kwargs.keys()  # Flag indicating if data was requested
 
-    # Validate inputs
-    if _data_id:
+    # If metadata is requested
+    if 'metadata' in kwargs.keys():
+        if kwargs['metadata']:
+            return metadata_from_h5(fp, tg)
+
+    # If data is requested
+    elif 'data_id' in kwargs.keys():
         data_id = kwargs['data_id']
-        assert(isinstance(data_id,(int,str,list,tuple))), "Error: data must be specified with strings or integers only."
-        if not isinstance(data_id,(int,str)):
-            assert(all([isinstance(d,(int,str)) for d in data_id])), "Error: data must be specified with strings or integers only."
+        assert(isinstance(data_id,(int,np.int_,str,list,tuple))), "Error: data must be specified with strings or integers only."
+        if not isinstance(data_id,(int,np.int_,str)):
+            assert(all([isinstance(d,(int,np.int_,str)) for d in data_id])), "Error: data must be specified with strings or integers only."
 
-    # Parse optional arguments
-    if 'mem' in kwargs.keys():
-        mem = kwargs['mem']
-        assert(mem in ('RAM','MEMMAP'))
-    else:
-        mem='RAM'
-    if 'binfactor' in kwargs.keys():
-        binfactor = kwargs['binfactor']
-        assert(isinstance(binfactor,int))
-    else:
-        binfactor=1
-    if 'dtype' in kwargs.keys():
-        bindtype = kwargs['dtype']
-        assert(isinstance(bindtype,type))
-    else:
-        bindtype = None
+        # Parse optional arguments
+        if 'mem' in kwargs.keys():
+            mem = kwargs['mem']
+            assert(mem in ('RAM','MEMMAP'))
+        else:
+            mem='RAM'
+        if 'binfactor' in kwargs.keys():
+            binfactor = kwargs['binfactor']
+            assert(isinstance(binfactor,(int,np.int_)))
+        else:
+            binfactor=1
+        if 'dtype' in kwargs.keys():
+            bindtype = kwargs['dtype']
+            assert(isinstance(bindtype,type))
+        else:
+            bindtype = None
 
-    # Perform requested operations
-    if not _data_id:
+        return get_data(fp,tg,data_id,mem,binfactor,bindtype)
+
+    # If no data is requested
+    else:
         print_py4DSTEM_file(fp,tg)
         return
-    else:
-        return get_data(fp,tg,data_id,mem,binfactor,bindtype)
+
+
+
+
+
+
+
 
 
 ############ Helper functions ############
 
-def print_py4DSTEM_file(fp,tg):
-    """ Accepts a fp to a valid py4DSTEM file and prints to screen the file contents.
+def print_py4DSTEM_file(filepath,tg):
+    """ Accepts a filepath to a valid py4DSTEM file and prints to screen the file contents.
     """
-    info = get_py4DSTEM_dataobject_info(fp,tg)
+    info = get_py4DSTEM_dataobject_info(filepath,tg)
 
-    version = get_py4DSTEM_version(fp, tg)
+    version = get_py4DSTEM_version(filepath, tg)
     print(f"py4DSTEM file version {version[0]}.{version[1]}.{version[2]}")
 
     print("{:10}{:18}{:24}{:54}".format('Index', 'Type', 'Shape', 'Name'))
     print("{:10}{:18}{:24}{:54}".format('-----', '----', '-----', '----'))
     for el in info:
         print("  {:8}{:18}{:24}{:54}".format(str(el['index']),str(el['type']),str(el['shape']),str(el['name'])))
-
     return
 
-def get_data(fp,tg,data_id,mem='RAM',binfactor=1,bindtype=None):
-    """ Accepts a fp to a valid py4DSTEM file and an int/str/list specifying data, and returns the data.
+def get_data(filepath,tg,data_id,mem='RAM',binfactor=1,bindtype=None):
+    """ Accepts a filepath to a valid py4DSTEM file and an int/str/list specifying data, and returns the data.
     """
-    if isinstance(data_id,int):
-        return get_data_from_int(fp,tg,data_id,mem=mem,binfactor=binfactor,bindtype=bindtype)
+    if isinstance(data_id,(int,np.int_)):
+        return get_data_from_int(filepath,tg,data_id,mem=mem,binfactor=binfactor,bindtype=bindtype)
     elif isinstance(data_id,str):
-        return get_data_from_str(fp,tg,data_id,mem=mem,binfactor=binfactor,bindtype=bindtype)
+        return get_data_from_str(filepath,tg,data_id,mem=mem,binfactor=binfactor,bindtype=bindtype)
     else:
-        return get_data_from_list(fp,tg,data_id)
+        return get_data_from_list(filepath,tg,data_id)
 
-def get_data_from_int(fp,tg,data_id,mem='RAM',binfactor=1,bindtype=None):
-    """ Accepts a fp to a valid py4DSTEM file and an integer specifying data, and returns the data.
+def get_data_from_int(filepath,tg,data_id,mem='RAM',binfactor=1,bindtype=None):
+    """ Accepts a filepath to a valid py4DSTEM file and an integer specifying data, and returns the data.
     """
-    assert(isinstance(data_id,int))
-    with h5py.File(fp,'r') as f:
+    assert(isinstance(data_id,(int,np.int_)))
+    with h5py.File(filepath,'r') as f:
         grp_dc = f[tg+'/data/datacubes/']
+        grp_cdc = f[tg+'/data/counted_datacubes/']
         grp_ds = f[tg+'/data/diffractionslices/']
         grp_rs = f[tg+'/data/realslices/']
         grp_pl = f[tg+'/data/pointlists/']
         grp_pla = f[tg+'/data/pointlistarrays/']
-        grps = [grp_dc,grp_ds,grp_rs,grp_pl,grp_pla]
+        grps = [grp_dc,grp_cdc,grp_ds,grp_rs,grp_pl,grp_pla]
 
         Ns = np.cumsum([len(grp.keys()) for grp in grps])
         i = np.nonzero(data_id<Ns)[0][0]
@@ -159,30 +171,32 @@ def get_data_from_int(fp,tg,data_id,mem='RAM',binfactor=1,bindtype=None):
             data = get_data_from_grp(grp_data,mem=mem,binfactor=binfactor,bindtype=bindtype)
 
     if mem == "MEMMAP":
-        f = h5py.File(fp,'r')
+        f = h5py.File(filepath,'r')
         grp_data = f[group_name]
         data = get_data_from_grp(grp_data,mem=mem,binfactor=binfactor,bindtype=bindtype)
 
     return data
 
-def get_data_from_str(fp,tg,data_id,mem='RAM',binfactor=1,bindtype=None):
-    """ Accepts a fp to a valid py4DSTEM file and a string specifying data, and returns the data.
+def get_data_from_str(filepath,tg,data_id,mem='RAM',binfactor=1,bindtype=None):
+    """ Accepts a filepath to a valid py4DSTEM file and a string specifying data, and returns the data.
     """
     assert(isinstance(data_id,str))
-    with h5py.File(fp,'r') as f:
+    with h5py.File(filepath,'r') as f:
         grp_dc = f[tg+'/data/datacubes/']
+        grp_cdc = f[tg+'/data/counted_datacubes/']
         grp_ds = f[tg+'/data/diffractionslices/']
         grp_rs = f[tg+'/data/realslices/']
         grp_pl = f[tg+'/data/pointlists/']
         grp_pla = f[tg+'/data/pointlistarrays/']
-        grps = [grp_dc,grp_ds,grp_rs,grp_pl,grp_pla]
+        grps = [grp_dc,grp_cdc,grp_ds,grp_rs,grp_pl,grp_pla]
 
         l_dc = list(grp_dc.keys())
+        l_cdc = list(grp_cdc.keys())
         l_ds = list(grp_ds.keys())
         l_rs = list(grp_rs.keys())
         l_pl = list(grp_pl.keys())
         l_pla = list(grp_pla.keys())
-        names = l_dc+l_ds+l_rs+l_pl+l_pla
+        names = l_dc+l_cdc+l_ds+l_rs+l_pl+l_pla
 
         inds = [i for i,name in enumerate(names) if name==data_id]
         assert(len(inds)!=0), "Error: no data named {} found.".format(data_id)
@@ -192,7 +206,6 @@ def get_data_from_str(fp,tg,data_id,mem='RAM',binfactor=1,bindtype=None):
         Ns = np.cumsum([len(grp.keys()) for grp in grps])
         i_grp = np.nonzero(ind<Ns)[0][0]
         grp = grps[i_grp]
-
         group_name = grp.name+'/'+data_id
 
         if mem == "RAM":
@@ -202,23 +215,23 @@ def get_data_from_str(fp,tg,data_id,mem='RAM',binfactor=1,bindtype=None):
     # if using MEMMAP, file cannot be accessed from the context manager
     # or else it will be closed before the data is accessed
     if mem == "MEMMAP":
-        f = h5py.File(fp,'r')
+        f = h5py.File(filepath,'r')
         grp_data = f[group_name]
         data = get_data_from_grp(grp_data,mem=mem,binfactor=binfactor,bindtype=bindtype)
 
     return data
 
-def get_data_from_list(fp,tg,data_id,mem='RAM',binfactor=1,bindtype=None):
-    """ Accepts a fp to a valid py4DSTEM file and a list or tuple specifying data, and returns the data.
+def get_data_from_list(filepath,tg,data_id,mem='RAM',binfactor=1,bindtype=None):
+    """ Accepts a filepath to a valid py4DSTEM file and a list or tuple specifying data, and returns the data.
     """
     assert(isinstance(data_id,(list,tuple)))
-    assert(all([isinstance(d,(int,str)) for d in data_id]))
+    assert(all([isinstance(d,(int,np.int_,str)) for d in data_id]))
     data = []
     for el in data_id:
-        if isinstance(el,int):
-            data.append(get_data_from_int(fp,tg,data_id=el,mem=mem,binfactor=binfactor,bindtype=bindtype))
+        if isinstance(el,(int,np.int_)):
+            data.append(get_data_from_int(filepath,tg,data_id=el,mem=mem,binfactor=binfactor,bindtype=bindtype))
         elif isinstance(el,str):
-            data.append(get_data_from_str(fp,tg,data_id=el,mem=mem,binfactor=binfactor,bindtype=bindtype))
+            data.append(get_data_from_str(filepath,tg,data_id=el,mem=mem,binfactor=binfactor,bindtype=bindtype))
         else:
             raise Exception("Data must be specified with strings or integers only.")
     return data
@@ -253,7 +266,7 @@ def get_datacube_from_grp(g,mem='RAM',binfactor=1,bindtype=None):
         data = np.array(g['data'])
     elif (mem, binfactor) == ("MEMMAP", 1):
         data = g['data']
-    
+
     name = g.name.split('/')[-1]
     return DataCube(data=data,name=name)
 
@@ -322,19 +335,13 @@ def get_pointlistarray_from_grp(g):
         and returns a PointListArray.
     """
     name = g.name.split('/')[-1]
-    #l = list(g)
-    print(g)
-    ar = np.array(g['data'])
-    shape = ar.shape
-    coord_names = ar[0,0].dtype.names
-    N = len(coord_names)
-    coord_types = [ar[0,0].dtype.fields[i][0] for i in coord_names]
-    coordinates = [(coord_names[i],coord_types[i]) for i in range(N)]
+    dset = g['data']
+    shape = g['data'].shape
+    coordinates = g['data'][0,0].dtype
     pla = PointListArray(coordinates=coordinates,shape=shape,name=name)
-    for (i,j) in tqdmnd(range(shape[0]),range(shape[1]),desc="Reading PointListArray",unit="PointList"):
-        pla.get_pointlist(i,j).add_dataarray(ar[i,j])
+    for (i,j) in tqdmnd(shape[0],shape[1],desc="Reading PointListArray",unit="PointList"):
+        pla.get_pointlist(i,j).add_dataarray(dset[i,j])
     return pla
 
 
 
-
